/* Sistemas Integrados TECNISA
   Planejamento Alimentao Industrial

Nome:  7791_PLASTPO_INTEGRA_RECEITA_REGRAS.SQL

Descrio: Realiza a reviso de receitas.

Parmetros de entrada:
   P_NRLISTA    => Nmero da lista onde est inserida as filiais agregadas
   P_CDPRATOINI => Cdigo do prato inicial
   P_CDPRATOFIN => Cdigo do prato final
   P_TIPOIMPORT => Tipo de importao (Acrescentar 'A', Sobrepor 'S')

Criao:
Leonardo Leo - 06/06/2006 */

CREATE OR REPLACE PROCEDURE INTEGRA_RECEITA_REGRAS
   (P_NRLISTA    IN VARCHAR2,
    P_CDPRATOINI IN VARCHAR2,
    P_CDPRATOFIN IN VARCHAR2,
    P_TIPOIMPORT IN VARCHAR2)
IS
   V_QTLIQUIDA      NUMBER(15, 5);
   V_QTBRUTA        NUMBER(15, 5);
   V_QTCALC         NUMBER(15, 5);
   V_QTCALCNUT      NUMBER(15, 5);
   V_CONTPROD       NUMBER;
   V_CONTRECEF      NUMBER;
   V_IDQTLIQUBRUT   VARCHAR2(1);
   V_PROD_JA_EXISTE VARCHAR2(10);
   V_S VARCHAR2(1);
   V_A VARCHAR2(1);
   V_P VARCHAR2(1);
BEGIN
   V_S := 'S';
   V_A := 'A';
   V_P := 'P';
   FOR C_PRATOSORIG IN
      (SELECT
          CDPRATO
       FROM
          PRATO
       WHERE
          CDFILAUXPRAT = V_P AND
          CDPRATO BETWEEN P_CDPRATOINI AND RPAD(P_CDPRATOFIN, 10, 'z') AND
          CDNVPRATO = (SELECT MAX(CDNVPRATO) FROM PRATO))
   LOOP
      FOR C_FILIAISAGREG IN
         (SELECT
             CDLISTA AS CDFILIAL
          FROM
             LISTA
          WHERE
             NRLISTA = P_NRLISTA)
      LOOP
         V_CONTPROD := 0;

         SELECT IDQTLIQUBRUT
         INTO   V_IDQTLIQUBRUT
         FROM   PARAMPLANEJA
         WHERE  CDFILIAL = C_FILIAISAGREG.CDFILIAL;


         SELECT COUNT(*)
            INTO V_CONTPROD
         FROM RECEFILI
         WHERE (CDFILIAL     = C_FILIAISAGREG.CDFILIAL) AND
               (CDFILAUXPRAT = V_P)                     AND
               (CDPRATO      = C_PRATOSORIG.CDPRATO);

         IF  V_CONTPROD > 0 THEN

            V_CONTPROD := 0;
            SELECT COUNT(*)
            INTO V_CONTPROD
            FROM
               REGRAREVRECE RR,
               RECEFILI RF,
               PRODUTO P
            WHERE
                RR.CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
                RR.IDAPLICAREG  = V_S                     AND
                RF.CDFILIAL     = RR.CDFILIAL             AND
                RF.CDFILAUXPRAT = V_P                     AND
                RF.CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                RR.CDPRODORIGRE = RF.CDPRODUTO            AND                                       
                RR.QTPRODORIGRE IN (DECODE(V_IDQTLIQUBRUT, 'B', ROUND(RF.QTPRODRECLOC, 5), ROUND(RF.QTBRUTALOUNR, 5)), 0) AND
                RF.CDPRODUTO    = P.CDPRODUTO             AND
                NVL(RF.SGUNIDADE, P.SGUNIDADE) = P.SGUNIDADE;

             /*S aplica a regra se pelo menos um produto da regra coincidir com algum produto da receita LOCAL */
             IF V_CONTPROD > 0 THEN

               /* OPO DE SOBREPOR */
               IF P_TIPOIMPORT = V_S THEN
                  DELETE FROM RECEFILI
                  WHERE
                     CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
                     CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                     CDFILAUXPRAT = V_P                     AND
                     NOT EXISTS(SELECT P.CDPRODUTO
                                FROM  REGRAREVRECE RR, PRODUTO P
                                WHERE
                                   RR.CDFILIAL         = RECEFILI.CDFILIAL    AND
                                   RR.IDAPLICAREG      = V_S                  AND
                                   RR.CDPRODORIGRE     = RECEFILI.CDPRODUTO   AND
                                   RR.QTPRODORIGRE IN (DECODE(V_IDQTLIQUBRUT, 'B', ROUND(RECEFILI.QTPRODRECLOC, 5), ROUND(RECEFILI.QTBRUTALOUNR, 5)), 0) AND
                                   RECEFILI.CDPRODUTO  = P.CDPRODUTO          AND
                                   NVL(RECEFILI.SGUNIDADE, P.SGUNIDADE) = P.SGUNIDADE);

               END IF;

               FOR C_UPDATEPROD IN (
                  SELECT
                     RF.CDPRODUTO,
                     RF.QTPRODRECLOC,
                     RF.VRPEFATCOCLO,
                     RF.VRPEFATCORLO,
                     RF.HRTEMPPREPLO,
                     RF.SGUNIDADE,
                     RF.QTBRUTALO,/*Deve possuir o mesmo valor que a Quantidade requerida */
                     RF.IDPRODBASELO,
                     RF.VRPEFATNUTLO,
                     RF.QTCALCNUTRLO,
                     RF.QTBRUTALOUNR,
                     RR.CDPRODDESTRE,
                     RR.QTPRODDESTRE
                  FROM
                     REGRAREVRECE RR,
                     RECEFILI RF,
                     PRODUTO P,
                     PRODFILI PF,
                     PRODUTO PV
                  WHERE
                     RR.CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
                     RR.IDAPLICAREG  = V_S                     AND
                     RF.CDFILIAL     = RR.CDFILIAL             AND
                     RF.CDFILAUXPRAT = V_P                     AND
                     RF.CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                     PF.CDFILIAL(+)  = RR.CDFILIAL             AND
                     PF.CDPRODUTO(+) = RR.CDPRODDESTRE         AND
                     NVL(PF.IDCNTRESTOQ, V_S) = V_S            AND
                     NVL(PF.IDPRODATIV, V_A) = V_A             AND
                     PV.CDPRODUTO    = NVL(RR.CDPRODDESTRE, RR.CDPRODORIGRE) AND
                     PV.IDPRODATIVPR = V_S                     AND
                     RR.CDPRODORIGRE = RF.CDPRODUTO            AND
                     RR.QTPRODORIGRE IN (DECODE(V_IDQTLIQUBRUT, 'B', ROUND(RF.QTPRODRECLOC, 5), ROUND(RF.QTBRUTALOUNR, 5)), 0) AND
                     RF.CDPRODUTO    = P.CDPRODUTO                AND
                     NVL(RF.SGUNIDADE, P.SGUNIDADE) = P.SGUNIDADE AND
                     NOT EXISTS (SELECT REC.CDPRODUTO
                                 FROM RECEFILI REC
                                 WHERE REC.CDFILIAL     = RF.CDFILIAL     AND
                                       REC.CDFILAUXPRAT = RF.CDFILAUXPRAT AND
                                       REC.CDPRATO      = RF.CDPRATO      AND
                                       REC.CDPRODUTO    = RR.CDPRODDESTRE AND
                                       RR.QTPRODDESTRE IN (DECODE(V_IDQTLIQUBRUT, 'B', ROUND(REC.QTPRODRECLOC, 5), ROUND(REC.QTBRUTALOUNR, 5)), 0) )

               )  LOOP




                  IF (C_UPDATEPROD.CDPRODDESTRE IS NULL) THEN
                     DELETE FROM RECEFILI
                     WHERE
                        (CDFILIAL     = C_FILIAISAGREG.CDFILIAL) AND
                        (CDFILAUXPRAT = V_P) AND
                        (CDPRATO      = C_PRATOSORIG.CDPRATO) AND
                        (CDPRODUTO    = C_UPDATEPROD.CDPRODUTO);
                  ELSE

                     /*VERIFICA SE O PRODUTO JA EXISTE NA RECEITA, CASO EXISTA, SERA IGNORADO, NAO REALIZANDO A SUBSTITUICAO PARA QUE NAO DE ERRO NA CHAVE PRIMARIA*/
                     V_PROD_JA_EXISTE := 'N';

                     BEGIN
                        SELECT 'S'
                          INTO V_PROD_JA_EXISTE
                          FROM RECEFILI
                         WHERE
                            (CDFILIAL     = C_FILIAISAGREG.CDFILIAL) AND
                            (CDFILAUXPRAT = V_P) AND
                            (CDPRATO      = C_PRATOSORIG.CDPRATO) AND
                            (CDPRODUTO    = C_UPDATEPROD.CDPRODDESTRE);
                     EXCEPTION
                        WHEN NO_DATA_FOUND THEN
                           V_PROD_JA_EXISTE := 'N';
                     END;
                     
                     IF C_UPDATEPROD.QTPRODDESTRE <> 0 THEN
                        V_QTLIQUIDA := 0;
                        V_QTBRUTA   := 0;
                        V_QTCALC    := 0;
                        V_QTCALCNUT := 0;

                        IF (C_UPDATEPROD.VRPEFATCOCLO IS NULL) OR (C_UPDATEPROD.VRPEFATCOCLO = 0) THEN
                           V_QTCALC := C_UPDATEPROD.QTPRODDESTRE;
                        ELSE
                           IF V_IDQTLIQUBRUT = 'B' THEN
                              V_QTCALC := C_UPDATEPROD.QTPRODDESTRE * (100 / (100 + C_UPDATEPROD.VRPEFATCOCLO));
                            ELSE
                              V_QTCALC := C_UPDATEPROD.QTPRODDESTRE / (100 / (100 + C_UPDATEPROD.VRPEFATCOCLO));
                           END IF;
                        END IF;

                        IF V_IDQTLIQUBRUT = 'B' THEN
                           V_QTLIQUIDA := C_UPDATEPROD.QTPRODDESTRE;
                           IF (C_UPDATEPROD.VRPEFATCORLO IS NULL) OR (C_UPDATEPROD.VRPEFATCORLO = 0) THEN
                              V_QTBRUTA := V_QTCALC;
                            ELSE
                              V_QTBRUTA := V_QTCALC * (100 / (100 - C_UPDATEPROD.VRPEFATCORLO));
                           END IF;
                        ELSE
                           V_QTBRUTA := C_UPDATEPROD.QTPRODDESTRE;
                           IF (C_UPDATEPROD.VRPEFATCORLO IS NULL) OR (C_UPDATEPROD.VRPEFATCORLO = 0) THEN
                              V_QTLIQUIDA := V_QTCALC;
                           ELSE
                              V_QTLIQUIDA := V_QTCALC / (100 / (100 - C_UPDATEPROD.VRPEFATCORLO));
                           END IF;
                        END IF;

                        IF (C_UPDATEPROD.VRPEFATNUTLO IS NULL) OR (C_UPDATEPROD.VRPEFATNUTLO = 0) THEN
                           V_QTCALCNUT := V_QTLIQUIDA;
                        ELSE
                          V_QTCALCNUT := (V_QTLIQUIDA * C_UPDATEPROD.VRPEFATNUTLO) / 100;
                        END IF;

                        IF V_PROD_JA_EXISTE <> 'S' THEN
                           UPDATE RECEFILI
                           SET CDPRODUTO    = C_UPDATEPROD.CDPRODDESTRE,
                               QTPRODRECLOC = V_QTLIQUIDA,
                               VRPEFATCOCLO = C_UPDATEPROD.VRPEFATCOCLO,
                               VRPEFATCORLO = C_UPDATEPROD.VRPEFATCORLO,
                               SGUNIDADE    = C_UPDATEPROD.SGUNIDADE,
                               QTUNIDESTLO  = V_QTBRUTA,
                               QTBRUTALO    = V_QTBRUTA,
                               IDPRODBASELO = C_UPDATEPROD.IDPRODBASELO,
                               DTULTATURECF = SYSDATE,
                               VRPEFATNUTLO = C_UPDATEPROD.VRPEFATNUTLO,
                               QTCALCNUTRLO = V_QTCALCNUT,
                               QTBRUTALOUNR = V_QTBRUTA
                           WHERE
                              (CDFILIAL     = C_FILIAISAGREG.CDFILIAL) AND
                              (CDFILAUXPRAT = V_P) AND
                              (CDPRATO      = C_PRATOSORIG.CDPRATO) AND
                              (CDPRODUTO    = C_UPDATEPROD.CDPRODUTO);
                        END IF;

                     ELSE
                        IF V_PROD_JA_EXISTE <> 'S' THEN
                           UPDATE RECEFILI
                           SET CDPRODUTO = C_UPDATEPROD.CDPRODDESTRE,
                               SGUNIDADE = C_UPDATEPROD.SGUNIDADE
                           WHERE
                              (CDFILIAL     = C_FILIAISAGREG.CDFILIAL) AND
                              (CDFILAUXPRAT = V_P) AND
                              (CDPRATO      = C_PRATOSORIG.CDPRATO) AND
                              (CDPRODUTO    = C_UPDATEPROD.CDPRODUTO);
                        END IF;
                     END IF;
                  END IF;

               END LOOP;
            END IF; /* REGRA APLICADA RECEITA LOCAL */


         ELSE  /* APLICAR NA RECEITA PADRAO PARA LOCAL*/


            SELECT COUNT(*)
            INTO V_CONTPROD
            FROM
               REGRAREVRECE RR,
               RECEPADR RP,
               PRODUTO P
            WHERE
               RR.CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
               RR.IDAPLICAREG  = V_S AND
               RP.CDFILAUXPRAT = V_P AND
               RP.CDPRATO      = C_PRATOSORIG.CDPRATO AND
               RR.CDPRODORIGRE = RP.CDPRODUTO AND
               RR.QTPRODORIGRE IN (DECODE(V_IDQTLIQUBRUT, 'B', ROUND(RP.QTPRODPRAT, 5), ROUND(RP.QTBRUTAPRUNR, 5)), 0) AND
               RP.CDPRODUTO    = P.CDPRODUTO  AND
               NVL(RP.SGUNIDADE, P.SGUNIDADE) = P.SGUNIDADE;

            /*S aplica a regra se pelo menos um produto da regra coincidir com algum produto da receita padro*/
            IF V_CONTPROD > 0 THEN
               /* OPO DE SOBREPOR */
               IF P_TIPOIMPORT = V_S THEN
                  DELETE FROM RECEFILI
                  WHERE
                     CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
                     CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                     CDFILAUXPRAT = V_P;

                  /*INSERE NA RECEFILI TODOS OS PRODUTOS DA RECEPADR QUE NO EXISTAM NA REGRAS*/
                  INSERT INTO RECEFILI
                     (CDFILAUXPRAT,
                      CDPRATO,
                      CDFILIAL,
                      CDPRODUTO,
                      QTPRODRECLOC,
                      VRPEFATCOCLO,
                      VRPEFATCORLO,
                      HRTEMPPREPLO,
                      SGUNIDADE,
                      QTUNIDESTLO,
                      QTBRUTALO,
                      IDPRODBASELO,
                      DTULTATURECF,
                      VRPEFATNUTLO,
                      QTCALCNUTRLO,
                      QTBRUTALOUNR)
                  (SELECT
                      V_P,
                      R.CDPRATO,
                      C_FILIAISAGREG.CDFILIAL,
                      R.CDPRODUTO,
                      R.QTPRODPRAT,
                      R.VRPEFATCOCPR,
                      R.VRPEFATCORPR,
                      R.HRTEMPPREPPR,
                      R.SGUNIDADE,
                      R.QTBRUTAPR,/*Deve possuir o mesmo valor que a Quantidade requerida*/
                      R.QTBRUTAPR,
                      R.IDPRODBASEPR,
                      SYSDATE,
                      R.VRPEFATNUTPR,
                      R.QTCALCNUTRPR,
                      R.QTBRUTAPRUNR
                   FROM
                      PRODUTO P,
                      PRODFILI PF,
                      RECEPADR R
                   WHERE
                      R.CDPRATO       = C_PRATOSORIG.CDPRATO    AND
                      R.CDFILAUXPRAT  = V_P                     AND
                      R.CDPRODUTO     = P.CDPRODUTO             AND
                      PF.CDFILIAL (+) = C_FILIAISAGREG.CDFILIAL AND
                      PF.CDPRODUTO(+) = R.CDPRODUTO             AND
                      NVL(PF.IDCNTRESTOQ, V_S) = V_S            AND
                      NVL(PF.IDPRODATIV, V_A) = V_A             AND
                      P.IDPRODATIVPR = V_S                      AND
                      NOT EXISTS (SELECT RE.CDPRODORIGRE
                                  FROM   REGRAREVRECE RE,
                                         PRODUTO PT
                                  WHERE  RE.CDFILIAL = C_FILIAISAGREG.CDFILIAL AND
                                         RE.IDAPLICAREG = V_S                  AND
                                         RE.CDPRODORIGRE = R.CDPRODUTO         AND
                                         RE.QTPRODORIGRE IN (DECODE(V_IDQTLIQUBRUT, 'B', ROUND(R.QTPRODPRAT, 5), ROUND(R.QTBRUTAPRUNR, 5)), 0) AND
                                         RE.CDPRODORIGRE = PT.CDPRODUTO        AND
                                         NVL(R.SGUNIDADE, PT.SGUNIDADE) = PT.SGUNIDADE));

                  /*RETORNA OS PRODUTOS QUE EXISTAM NA RECEPADR, EXISTAM NA REGRAS E QUE O PRODUTO DE
                    DESTINO NO EXISTA NA RECEFILI*/
                  FOR C_PRODREGRAS IN
                    (SELECT
                        RC.CDPRATO,
                        RC.CDPRODUTO,
                        RC.QTPRODPRAT,
                        RC.VRPEFATCOCPR,
                        RC.VRPEFATCORPR,
                        RC.HRTEMPPREPPR,
                        RC.SGUNIDADE,
                        RC.QTUNIDESTPR,
                        RC.QTBRUTAPR,
                        RC.IDPRODBASEPR,
                        RC.VRPEFATNUTPR,
                        RC.QTCALCNUTRPR,
                        RC.QTBRUTAPRUNR,
                        R.CDPRODDESTRE,
                        R.QTPRODDESTRE
                     FROM
                        PRODUTO P,
                        PRODFILI PF,
                        RECEPADR RC,
                        REGRAREVRECE R
                     WHERE
                        RC.CDPRODUTO    = P.CDPRODUTO             AND
                        PF.CDFILIAL (+) = C_FILIAISAGREG.CDFILIAL AND
                        PF.CDPRODUTO(+) = RC.CDPRODUTO            AND
                        NVL(PF.IDCNTRESTOQ, V_S) = V_S            AND
                        NVL(PF.IDPRODATIV, V_A) = V_A             AND
                        P.IDPRODATIVPR  = V_S                     AND
                        RC.CDFILAUXPRAT = V_P                     AND
                        RC.CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                        R.CDFILIAL      = C_FILIAISAGREG.CDFILIAL AND
                        R.IDAPLICAREG   = V_S           AND
                        R.CDPRODDESTRE IS NOT NULL      AND
                        R.CDPRODORIGRE  = RC.CDPRODUTO  AND
                        R.QTPRODORIGRE  = DECODE(V_IDQTLIQUBRUT, 'B', ROUND(RC.QTPRODPRAT, 5), ROUND(RC.QTBRUTAPRUNR, 5)) AND
                        NVL(RC.SGUNIDADE, P.SGUNIDADE) = P.SGUNIDADE AND
                        NOT EXISTS (SELECT RF.CDPRODUTO
                                    FROM   RECEFILI RF
                                    WHERE  RF.CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
                                           RF.CDFILAUXPRAT = V_P                     AND
                                           RF.CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                                           RF.CDPRODUTO    = R.CDPRODDESTRE)
                     ORDER BY P.CDARVPROD)
                     /*SE EXISTIREM MAIS DE UM PRODUTO NA REGRA COM O MESMO PRODUTO DE DESTINO, ENTO SER APLICADO A
                       REGRA NO PRODUTO COM MENOR CDIGO DE RVORE
                       OBS.: ISSO FOI FEITO PARA EVITAR ERRO DE PK DUPLICADA NA RECEFILI*/
                  LOOP
                     BEGIN
                        SELECT CDPRODUTO
                        INTO   V_PROD_JA_EXISTE
                        FROM   RECEFILI
                        WHERE  CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
                               CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                               CDFILAUXPRAT = V_P AND
                               CDPRODUTO    = C_PRODREGRAS.CDPRODDESTRE;
                        /*SE GERAR EXCEO, ENTO O PRODUTO NO EXISTE NA RECEFILI E PODE SER INSERIDO*/
                     EXCEPTION
                        WHEN NO_DATA_FOUND THEN
                           V_QTLIQUIDA := 0;
                           V_QTBRUTA   := 0;
                           V_QTCALC    := 0;
                           V_QTCALCNUT := 0;

                           IF (C_PRODREGRAS.VRPEFATCOCPR IS NULL) OR (C_PRODREGRAS.VRPEFATCOCPR = 0) THEN
                              V_QTCALC := C_PRODREGRAS.QTPRODDESTRE;
                           ELSE
                              IF V_IDQTLIQUBRUT = 'B' THEN
                                 V_QTCALC := C_PRODREGRAS.QTPRODDESTRE * (100 / (100 + C_PRODREGRAS.VRPEFATCOCPR));
                              ELSE
                                 V_QTCALC := C_PRODREGRAS.QTPRODDESTRE / (100 / (100 + C_PRODREGRAS.VRPEFATCOCPR));
                              END IF;
                           END IF;

                           IF V_IDQTLIQUBRUT = 'B' THEN
                              V_QTLIQUIDA := C_PRODREGRAS.QTPRODDESTRE;
                              IF (C_PRODREGRAS.VRPEFATCORPR IS NULL) OR (C_PRODREGRAS.VRPEFATCORPR = 0) THEN
                                 V_QTBRUTA := V_QTCALC;
                              ELSE
                                 V_QTBRUTA := V_QTCALC * (100 / (100 - C_PRODREGRAS.VRPEFATCORPR));
                              END IF;
                           ELSE
                              V_QTBRUTA := C_PRODREGRAS.QTPRODDESTRE;
                              IF (C_PRODREGRAS.VRPEFATCORPR IS NULL) OR (C_PRODREGRAS.VRPEFATCORPR = 0) THEN
                                 V_QTLIQUIDA := V_QTCALC;
                              ELSE
                                 V_QTLIQUIDA := V_QTCALC / (100 / (100 - C_PRODREGRAS.VRPEFATCORPR));
                              END IF;
                           END IF;

                           IF (C_PRODREGRAS.VRPEFATNUTPR IS NULL) OR (C_PRODREGRAS.VRPEFATNUTPR = 0) THEN
                              V_QTCALCNUT := V_QTLIQUIDA;
                           ELSE
                              V_QTCALCNUT := (V_QTLIQUIDA * C_PRODREGRAS.VRPEFATNUTPR) / 100;
                           END IF;

                           INSERT INTO RECEFILI
                              (CDFILAUXPRAT,
                               CDPRATO,
                               CDFILIAL,
                               CDPRODUTO,
                               QTPRODRECLOC, /*Quantidade lquida*/
                               VRPEFATCOCLO,
                               VRPEFATCORLO,
                               HRTEMPPREPLO,
                               SGUNIDADE,
                               QTUNIDESTLO,
                               QTBRUTALO,    /*Quantidade requerida*/
                               IDPRODBASELO,
                               DTULTATURECF,
                               VRPEFATNUTLO,
                               QTCALCNUTRLO,
                               QTBRUTALOUNR) /*Quantidade bruta*/
                           VALUES
                              (V_P,
                               C_PRODREGRAS.CDPRATO,
                               C_FILIAISAGREG.CDFILIAL,
                               C_PRODREGRAS.CDPRODDESTRE,
                               V_QTLIQUIDA,
                               C_PRODREGRAS.VRPEFATCOCPR,
                               C_PRODREGRAS.VRPEFATCORPR,
                               C_PRODREGRAS.HRTEMPPREPPR,
                               C_PRODREGRAS.SGUNIDADE,
                               V_QTBRUTA,/*Deve possuir o mesmo valor que a Quantidade requerida*/
                               V_QTBRUTA,
                               C_PRODREGRAS.IDPRODBASEPR,
                               SYSDATE,
                               C_PRODREGRAS.VRPEFATNUTPR,
                               V_QTCALCNUT,
                               V_QTBRUTA);
                     END;
                  END LOOP;

                  /*RETORNA TODOS OS PRODUTOS QUE ESTO ZERADOS NA REGRA, QUE EXISTAM NA RECEPADR E QUE J
                    NO TENHA SIDO APLICADA REGRA AO PRODUTO*/
                  FOR C_REGRAS_PROD_ZERO IN
                    (SELECT
                        RP.CDPRATO,
                        RP.CDPRODUTO,
                        RP.QTPRODPRAT,
                        RP.VRPEFATCOCPR,
                        RP.VRPEFATCORPR,
                        RP.HRTEMPPREPPR,
                        RP.SGUNIDADE,
                        RP.QTUNIDESTPR,
                        RP.QTBRUTAPR,
                        RP.IDPRODBASEPR,
                        RP.VRPEFATNUTPR,
                        RP.QTCALCNUTRPR,
                        RP.QTBRUTAPRUNR,
                        R.CDPRODDESTRE,
                        R.QTPRODDESTRE
                     FROM
                        PRODUTO P,
                        PRODFILI PF,
                        RECEPADR RP,
                        REGRAREVRECE R
                     WHERE
                        RP.CDPRODUTO    = P.CDPRODUTO             AND
                        PF.CDFILIAL (+) = C_FILIAISAGREG.CDFILIAL AND
                        PF.CDPRODUTO(+) = RP.CDPRODUTO            AND
                        NVL(PF.IDCNTRESTOQ, V_S) = V_S            AND
                        NVL(PF.IDPRODATIV, V_A) = V_A             AND
                        P.IDPRODATIVPR  = V_S                     AND
                        RP.CDFILAUXPRAT = V_P                     AND
                        RP.CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                        R.CDFILIAL      = C_FILIAISAGREG.CDFILIAL AND
                        R.IDAPLICAREG   = V_S           AND
                        R.CDPRODDESTRE IS NOT NULL      AND
                        R.CDPRODORIGRE  = RP.CDPRODUTO  AND
                        R.QTPRODORIGRE  = 0 AND
                        NVL(RP.SGUNIDADE, P.SGUNIDADE) = P.SGUNIDADE AND
                        NOT EXISTS (SELECT
                                       R.CDPRODORIGRE
                                    FROM
                                       PRODUTO P,
                                       PRODFILI PF,
                                       RECEPADR RC,
                                       REGRAREVRECE R
                                    WHERE
                                       RC.CDPRODUTO    = P.CDPRODUTO             AND
                                       PF.CDFILIAL (+) = C_FILIAISAGREG.CDFILIAL AND
                                       PF.CDPRODUTO(+) = RC.CDPRODUTO            AND
                                       NVL(PF.IDCNTRESTOQ, V_S) = V_S            AND
                                       NVL(PF.IDPRODATIV, V_A) = V_A             AND
                                       P.IDPRODATIVPR  = V_S                     AND
                                       RC.CDFILAUXPRAT = V_P                     AND
                                       RC.CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                                       R.CDFILIAL      = C_FILIAISAGREG.CDFILIAL AND
                                       R.IDAPLICAREG   = V_S           AND
                                       R.CDPRODORIGRE  = RC.CDPRODUTO  AND
                                       R.QTPRODORIGRE  = DECODE(V_IDQTLIQUBRUT, 'B', ROUND(RC.QTPRODPRAT, 5), ROUND(RC.QTBRUTAPRUNR, 5)) AND
                                       NVL(RC.SGUNIDADE, P.SGUNIDADE) = P.SGUNIDADE AND
                                       R.CDPRODORIGRE  = RP.CDPRODUTO) AND
                        NOT EXISTS (SELECT RF.CDPRODUTO
                                    FROM   RECEFILI RF
                                    WHERE  RF.CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
                                           RF.CDFILAUXPRAT = V_P                     AND
                                           RF.CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                                           RF.CDPRODUTO    = R.CDPRODDESTRE))
                  LOOP
                     BEGIN
                        SELECT CDPRODUTO
                        INTO   V_PROD_JA_EXISTE
                        FROM   RECEFILI
                        WHERE  CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
                               CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                               CDFILAUXPRAT = V_P AND
                               CDPRODUTO    = C_REGRAS_PROD_ZERO.CDPRODDESTRE;
                        /*SE GERAR EXCEO, ENTO O PRODUTO NO EXISTE NA RECEFILI E PODE SER INSERIDO*/
                     EXCEPTION
                        WHEN NO_DATA_FOUND THEN
                           IF C_REGRAS_PROD_ZERO.QTPRODDESTRE <> 0 THEN
                              V_QTLIQUIDA := 0;
                              V_QTBRUTA   := 0;
                              V_QTCALC    := 0;
                              V_QTCALCNUT := 0;

                              IF (C_REGRAS_PROD_ZERO.VRPEFATCOCPR IS NULL) OR (C_REGRAS_PROD_ZERO.VRPEFATCOCPR = 0) THEN
                                 V_QTCALC := C_REGRAS_PROD_ZERO.QTPRODDESTRE;
                              ELSE
                                 IF V_IDQTLIQUBRUT = 'B' THEN
                                    V_QTCALC := C_REGRAS_PROD_ZERO.QTPRODDESTRE * (100 / (100 + C_REGRAS_PROD_ZERO.VRPEFATCOCPR));
                                 ELSE
                                    V_QTCALC := C_REGRAS_PROD_ZERO.QTPRODDESTRE / (100 / (100 + C_REGRAS_PROD_ZERO.VRPEFATCOCPR));
                                 END IF;
                              END IF;

                              IF V_IDQTLIQUBRUT = 'B' THEN
                                 V_QTLIQUIDA := C_REGRAS_PROD_ZERO.QTPRODDESTRE;
                                 IF (C_REGRAS_PROD_ZERO.VRPEFATCORPR IS NULL) OR (C_REGRAS_PROD_ZERO.VRPEFATCORPR = 0) THEN
                                    V_QTBRUTA := V_QTCALC;
                                 ELSE
                                    V_QTBRUTA := V_QTCALC * (100 / (100 - C_REGRAS_PROD_ZERO.VRPEFATCORPR));
                                 END IF;
                              ELSE
                                 V_QTBRUTA := C_REGRAS_PROD_ZERO.QTPRODDESTRE;
                                 IF (C_REGRAS_PROD_ZERO.VRPEFATCORPR IS NULL) OR (C_REGRAS_PROD_ZERO.VRPEFATCORPR = 0) THEN
                                    V_QTLIQUIDA := V_QTCALC;
                                 ELSE
                                    V_QTLIQUIDA := V_QTCALC / (100 / (100 - C_REGRAS_PROD_ZERO.VRPEFATCORPR));
                                 END IF;
                              END IF;

                              IF (C_REGRAS_PROD_ZERO.VRPEFATNUTPR IS NULL) OR (C_REGRAS_PROD_ZERO.VRPEFATNUTPR = 0) THEN
                                 V_QTCALCNUT := V_QTLIQUIDA;
                              ELSE
                                 V_QTCALCNUT := (V_QTLIQUIDA * C_REGRAS_PROD_ZERO.VRPEFATNUTPR) / 100;
                              END IF;
                           ELSE
                              V_QTLIQUIDA := C_REGRAS_PROD_ZERO.QTPRODPRAT;
                              V_QTBRUTA   := C_REGRAS_PROD_ZERO.QTBRUTAPR;
                              V_QTCALCNUT := C_REGRAS_PROD_ZERO.QTCALCNUTRPR;
                           END IF;

                           INSERT INTO RECEFILI
                              (CDFILAUXPRAT,
                               CDPRATO,
                               CDFILIAL,
                               CDPRODUTO,
                               QTPRODRECLOC, /*Quantidade lquida*/
                               VRPEFATCOCLO,
                               VRPEFATCORLO,
                               HRTEMPPREPLO,
                               SGUNIDADE,
                               QTUNIDESTLO,
                               QTBRUTALO,    /*Quantidade requerida*/
                               IDPRODBASELO,
                               DTULTATURECF,
                               VRPEFATNUTLO,
                               QTCALCNUTRLO,
                               QTBRUTALOUNR) /*Quantidade bruta*/
                           VALUES
                              (V_P,
                               C_REGRAS_PROD_ZERO.CDPRATO,
                               C_FILIAISAGREG.CDFILIAL,
                               C_REGRAS_PROD_ZERO.CDPRODDESTRE,
                               V_QTLIQUIDA,
                               C_REGRAS_PROD_ZERO.VRPEFATCOCPR,
                               C_REGRAS_PROD_ZERO.VRPEFATCORPR,
                               C_REGRAS_PROD_ZERO.HRTEMPPREPPR,
                               C_REGRAS_PROD_ZERO.SGUNIDADE,
                               V_QTBRUTA,/*Deve possuir o mesmo valor que a Quantidade requerida*/
                               V_QTBRUTA,
                               C_REGRAS_PROD_ZERO.IDPRODBASEPR,
                               SYSDATE,
                               C_REGRAS_PROD_ZERO.VRPEFATNUTPR,
                               V_QTCALCNUT,
                               V_QTBRUTA);
                     END;
                  END LOOP;

               ELSE /* OPO DE ACRESCENTAR ****************************************************************
                       INSERE OS PRODUTOS QUE EXISTAM NA RECEPADR, NO EXISTAM NA RECEFILI E QUE NO EXISTAM
                       NA REGRAS*/
                  INSERT INTO RECEFILI
                     (CDFILAUXPRAT,
                      CDPRATO,
                      CDFILIAL,
                      CDPRODUTO,
                      QTPRODRECLOC,
                      VRPEFATCOCLO,
                      VRPEFATCORLO,
                      HRTEMPPREPLO,
                      SGUNIDADE,
                      QTUNIDESTLO,
                      QTBRUTALO,
                      IDPRODBASELO,
                      DTULTATURECF,
                      VRPEFATNUTLO,
                      QTCALCNUTRLO,
                      QTBRUTALOUNR)
                  (SELECT
                      V_P,
                      R.CDPRATO,
                      C_FILIAISAGREG.CDFILIAL,
                      R.CDPRODUTO,
                      R.QTPRODPRAT,
                      R.VRPEFATCOCPR,
                      R.VRPEFATCORPR,
                      R.HRTEMPPREPPR,
                      R.SGUNIDADE,
                      R.QTBRUTAPR,/*Deve possuir o mesmo valor que a Quantidade requerida*/
                      R.QTBRUTAPR,
                      R.IDPRODBASEPR,
                      SYSDATE,
                      R.VRPEFATNUTPR,
                      R.QTCALCNUTRPR,
                      R.QTBRUTAPRUNR
                   FROM
                      PRODUTO P,
                      PRODFILI PF,
                      RECEPADR R
                   WHERE
                      R.CDPRATO       = C_PRATOSORIG.CDPRATO    AND
                      R.CDFILAUXPRAT  = V_P                     AND
                      R.CDPRODUTO     = P.CDPRODUTO             AND
                      PF.CDFILIAL (+) = C_FILIAISAGREG.CDFILIAL AND
                      PF.CDPRODUTO(+) = R.CDPRODUTO             AND
                      NVL(PF.IDCNTRESTOQ, V_S) = V_S            AND
                      NVL(PF.IDPRODATIV, V_A) = V_A             AND
                      P.IDPRODATIVPR = V_S                      AND
                      NOT EXISTS (SELECT RF.CDPRODUTO
                                  FROM   RECEFILI RF
                                  WHERE  RF.CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
                                         RF.CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                                         RF.CDFILAUXPRAT = V_P                     AND
                                         RF.CDPRODUTO    = R.CDPRODUTO)            AND
                      NOT EXISTS (SELECT RE.CDPRODORIGRE
                                  FROM   REGRAREVRECE RE,
                                         PRODUTO P
                                  WHERE  RE.CDFILIAL = C_FILIAISAGREG.CDFILIAL AND
                                         RE.IDAPLICAREG = V_S                  AND
                                         RE.CDPRODORIGRE = R.CDPRODUTO         AND
                                         RE.QTPRODORIGRE IN (DECODE(V_IDQTLIQUBRUT, 'B', ROUND(R.QTPRODPRAT, 5), ROUND(R.QTBRUTAPRUNR, 5)), 0) AND
                                         RE.CDPRODORIGRE = P.CDPRODUTO AND
                                         NVL(R.SGUNIDADE, P.SGUNIDADE) = P.SGUNIDADE));

                  /*RETORNA OS PRODUTOS QUE EXISTAM NA RECEPADR, EXISTAM NA REGRAS E QUE O PRODUTO DE
                    ORIGEM E DESTINO NO EXISTA NA RECEFILI*/
                  FOR C_PRODREGRAS_ACRES IN
                    (SELECT
                        RC.CDPRATO,
                        RC.CDPRODUTO,
                        RC.QTPRODPRAT,
                        RC.VRPEFATCOCPR,
                        RC.VRPEFATCORPR,
                        RC.HRTEMPPREPPR,
                        RC.SGUNIDADE,
                        RC.QTUNIDESTPR,
                        RC.QTBRUTAPR,
                        RC.IDPRODBASEPR,
                        RC.VRPEFATNUTPR,
                        RC.QTCALCNUTRPR,
                        RC.QTBRUTAPRUNR,
                        R.CDPRODDESTRE,
                        R.QTPRODDESTRE
                     FROM
                        PRODUTO P,
                        PRODFILI PF,
                        RECEPADR RC,
                        REGRAREVRECE R
                     WHERE
                        RC.CDPRODUTO    = P.CDPRODUTO             AND
                        PF.CDFILIAL (+) = C_FILIAISAGREG.CDFILIAL AND
                        PF.CDPRODUTO(+) = RC.CDPRODUTO            AND
                        NVL(PF.IDCNTRESTOQ, V_S) = V_S            AND
                        NVL(PF.IDPRODATIV, V_A) = V_A             AND
                        P.IDPRODATIVPR  = V_S                     AND
                        RC.CDFILAUXPRAT = V_P                     AND
                        RC.CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                        R.CDFILIAL      = C_FILIAISAGREG.CDFILIAL AND
                        R.IDAPLICAREG   = V_S           AND
                        R.CDPRODDESTRE IS NOT NULL      AND
                        R.CDPRODORIGRE  = RC.CDPRODUTO  AND
                        R.QTPRODORIGRE  = DECODE(V_IDQTLIQUBRUT, 'B', ROUND(RC.QTPRODPRAT, 5), ROUND(RC.QTBRUTAPRUNR, 5)) AND
                        NVL(RC.SGUNIDADE, P.SGUNIDADE) = P.SGUNIDADE AND
                        NOT EXISTS (SELECT RF.CDPRODUTO
                                    FROM   RECEFILI RF
                                    WHERE  RF.CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
                                           RF.CDFILAUXPRAT = V_P                     AND
                                           RF.CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                                           RF.CDPRODUTO    = R.CDPRODORIGRE)         AND
                        NOT EXISTS (SELECT RF.CDPRODUTO
                                    FROM   RECEFILI RF
                                    WHERE  RF.CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
                                           RF.CDFILAUXPRAT = V_P                     AND
                                           RF.CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                                           RF.CDPRODUTO    = R.CDPRODDESTRE)
                     ORDER BY P.CDARVPROD)
                     /*SE EXISTIREM MAIS DE UM PRODUTO NA REGRA COM O MESMO PRODUTO DE DESTINO, ENTO SER APLICADO A
                       REGRA NO PRODUTO COM MENOR CDIGO DE RVORE
                       OBS.: ISSO FOI FEITO PARA EVITAR ERRO DE PK DUPLICADA NA RECEFILI*/
                  LOOP
                     BEGIN
                        SELECT CDPRODUTO
                        INTO   V_PROD_JA_EXISTE
                        FROM   RECEFILI
                        WHERE  CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
                               CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                               CDFILAUXPRAT = V_P AND
                               CDPRODUTO    = C_PRODREGRAS_ACRES.CDPRODDESTRE;
                        /*SE GERAR EXCEO, ENTO O PRODUTO NO EXISTE NA RECEFILI E PODE SER INSERIDO*/
                     EXCEPTION
                        WHEN NO_DATA_FOUND THEN
                           V_QTLIQUIDA := 0;
                           V_QTBRUTA   := 0;
                           V_QTCALC    := 0;
                           V_QTCALCNUT := 0;

                           IF (C_PRODREGRAS_ACRES.VRPEFATCOCPR IS NULL) OR (C_PRODREGRAS_ACRES.VRPEFATCOCPR = 0) THEN
                              V_QTCALC := C_PRODREGRAS_ACRES.QTPRODDESTRE;
                           ELSE
                              IF V_IDQTLIQUBRUT = 'B' THEN
                                 V_QTCALC := C_PRODREGRAS_ACRES.QTPRODDESTRE * (100 / (100 + C_PRODREGRAS_ACRES.VRPEFATCOCPR));
                              ELSE
                                 V_QTCALC := C_PRODREGRAS_ACRES.QTPRODDESTRE / (100 / (100 + C_PRODREGRAS_ACRES.VRPEFATCOCPR));
                              END IF;
                           END IF;

                           IF V_IDQTLIQUBRUT = 'B' THEN
                              V_QTLIQUIDA := C_PRODREGRAS_ACRES.QTPRODDESTRE;
                              IF (C_PRODREGRAS_ACRES.VRPEFATCORPR IS NULL) OR (C_PRODREGRAS_ACRES.VRPEFATCORPR = 0) THEN
                                 V_QTBRUTA := V_QTCALC;
                              ELSE
                                 V_QTBRUTA := V_QTCALC * (100 / (100 - C_PRODREGRAS_ACRES.VRPEFATCORPR));
                              END IF;
                           ELSE
                              V_QTBRUTA := C_PRODREGRAS_ACRES.QTPRODDESTRE;
                              IF (C_PRODREGRAS_ACRES.VRPEFATCORPR IS NULL) OR (C_PRODREGRAS_ACRES.VRPEFATCORPR = 0) THEN
                                 V_QTLIQUIDA := V_QTCALC;
                              ELSE
                                 V_QTLIQUIDA := V_QTCALC / (100 / (100 - C_PRODREGRAS_ACRES.VRPEFATCORPR));
                              END IF;
                           END IF;

                           IF (C_PRODREGRAS_ACRES.VRPEFATNUTPR IS NULL) OR (C_PRODREGRAS_ACRES.VRPEFATNUTPR = 0) THEN
                              V_QTCALCNUT := V_QTLIQUIDA;
                           ELSE
                              V_QTCALCNUT := (V_QTLIQUIDA * C_PRODREGRAS_ACRES.VRPEFATNUTPR) / 100;
                           END IF;

                           INSERT INTO RECEFILI
                              (CDFILAUXPRAT,
                               CDPRATO,
                               CDFILIAL,
                               CDPRODUTO,
                               QTPRODRECLOC, /*Quantidade lquida*/
                               VRPEFATCOCLO,
                               VRPEFATCORLO,
                               HRTEMPPREPLO,
                               SGUNIDADE,
                               QTUNIDESTLO,
                               QTBRUTALO,    /*Quantidade requerida*/
                               IDPRODBASELO,
                               DTULTATURECF,
                               VRPEFATNUTLO,
                               QTCALCNUTRLO,
                               QTBRUTALOUNR) /*Quantidade bruta*/
                           VALUES
                              (V_P,
                               C_PRODREGRAS_ACRES.CDPRATO,
                               C_FILIAISAGREG.CDFILIAL,
                               C_PRODREGRAS_ACRES.CDPRODDESTRE,
                               V_QTLIQUIDA,
                               C_PRODREGRAS_ACRES.VRPEFATCOCPR,
                               C_PRODREGRAS_ACRES.VRPEFATCORPR,
                               C_PRODREGRAS_ACRES.HRTEMPPREPPR,
                               C_PRODREGRAS_ACRES.SGUNIDADE,
                               V_QTBRUTA,/*Deve possuir o mesmo valor que a Quantidade requerida*/
                               V_QTBRUTA,
                               C_PRODREGRAS_ACRES.IDPRODBASEPR,
                               SYSDATE,
                               C_PRODREGRAS_ACRES.VRPEFATNUTPR,
                               V_QTCALCNUT,
                               V_QTBRUTA);
                     END;
                  END LOOP;

                  /*RETORNA OS PRODUTOS QUE EXISTAM NA RECEPADR, EXISTAM NA REGRAS COM QUANTIDADE ZERO E
                    QUE O PRODUTO DE ORIGEM E DESTINO NO EXISTA NA RECEFILI*/
                  FOR C_PRODREGRAS_ACRESZERO IN
                    (SELECT
                        RP.CDPRATO,
                        RP.CDPRODUTO,
                        RP.QTPRODPRAT,
                        RP.VRPEFATCOCPR,
                        RP.VRPEFATCORPR,
                        RP.HRTEMPPREPPR,
                        RP.SGUNIDADE,
                        RP.QTUNIDESTPR,
                        RP.QTBRUTAPR,
                        RP.IDPRODBASEPR,
                        RP.VRPEFATNUTPR,
                        RP.QTCALCNUTRPR,
                        RP.QTBRUTAPRUNR,
                        R.CDPRODDESTRE,
                        R.QTPRODDESTRE
                     FROM
                        PRODUTO P,
                        PRODFILI PF,
                        RECEPADR RP,
                        REGRAREVRECE R
                     WHERE
                        RP.CDPRODUTO    = P.CDPRODUTO             AND
                        PF.CDFILIAL (+) = C_FILIAISAGREG.CDFILIAL AND
                        PF.CDPRODUTO(+) = RP.CDPRODUTO            AND
                        NVL(PF.IDCNTRESTOQ, V_S) = V_S            AND
                        NVL(PF.IDPRODATIV, V_A) = V_A             AND
                        P.IDPRODATIVPR  = V_S                     AND
                        RP.CDFILAUXPRAT = V_P                     AND
                        RP.CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                        R.CDFILIAL      = C_FILIAISAGREG.CDFILIAL AND
                        R.IDAPLICAREG   = V_S           AND
                        R.CDPRODDESTRE IS NOT NULL      AND
                        R.CDPRODORIGRE  = RP.CDPRODUTO  AND
                        R.QTPRODORIGRE  = 0 AND
                        NVL(RP.SGUNIDADE, P.SGUNIDADE) = P.SGUNIDADE AND
                        NOT EXISTS (SELECT
                                       R.CDPRODORIGRE
                                    FROM
                                       PRODUTO P,
                                       PRODFILI PF,
                                       RECEPADR RC,
                                       REGRAREVRECE R
                                    WHERE
                                       RC.CDPRODUTO    = P.CDPRODUTO             AND
                                       PF.CDFILIAL (+) = C_FILIAISAGREG.CDFILIAL AND
                                       PF.CDPRODUTO(+) = RC.CDPRODUTO            AND
                                       NVL(PF.IDCNTRESTOQ, V_S) = V_S            AND
                                       NVL(PF.IDPRODATIV, V_A) = V_A             AND
                                       P.IDPRODATIVPR  = V_S                     AND
                                       RC.CDFILAUXPRAT = V_P                     AND
                                       RC.CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                                       R.CDFILIAL      = C_FILIAISAGREG.CDFILIAL AND
                                       R.IDAPLICAREG   = V_S           AND
                                       R.CDPRODORIGRE  = RC.CDPRODUTO  AND
                                       R.QTPRODORIGRE  = DECODE(V_IDQTLIQUBRUT, 'B', ROUND(RC.QTPRODPRAT, 5), ROUND(RC.QTBRUTAPRUNR, 5)) AND
                                       NVL(RC.SGUNIDADE, P.SGUNIDADE) = P.SGUNIDADE AND
                                       R.CDPRODORIGRE  = RP.CDPRODUTO) AND
                        NOT EXISTS (SELECT RF.CDPRODUTO
                                    FROM   RECEFILI RF
                                    WHERE  RF.CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
                                           RF.CDFILAUXPRAT = V_P                     AND
                                           RF.CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                                           RF.CDPRODUTO    = R.CDPRODORIGRE)         AND
                        NOT EXISTS (SELECT RF.CDPRODUTO
                                    FROM   RECEFILI RF
                                    WHERE  RF.CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
                                           RF.CDFILAUXPRAT = V_P                     AND
                                           RF.CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                                           RF.CDPRODUTO    = R.CDPRODDESTRE)
                     ORDER BY P.CDARVPROD)
                  LOOP
                     BEGIN
                        SELECT CDPRODUTO
                        INTO   V_PROD_JA_EXISTE
                        FROM   RECEFILI
                        WHERE  CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
                               CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                               CDFILAUXPRAT = V_P AND
                               CDPRODUTO    = C_PRODREGRAS_ACRESZERO.CDPRODDESTRE;
                        /*SE GERAR EXCEO, ENTO O PRODUTO NO EXISTE NA RECEFILI E PODE SER INSERIDO*/
                     EXCEPTION
                        WHEN NO_DATA_FOUND THEN
                           IF C_PRODREGRAS_ACRESZERO.QTPRODDESTRE <> 0 THEN
                              V_QTLIQUIDA := 0;
                              V_QTBRUTA   := 0;
                              V_QTCALC    := 0;
                              V_QTCALCNUT := 0;

                              IF (C_PRODREGRAS_ACRESZERO.VRPEFATCOCPR IS NULL) OR (C_PRODREGRAS_ACRESZERO.VRPEFATCOCPR = 0) THEN
                                 V_QTCALC := C_PRODREGRAS_ACRESZERO.QTPRODDESTRE;
                              ELSE
                                 IF V_IDQTLIQUBRUT = 'B' THEN
                                    V_QTCALC := C_PRODREGRAS_ACRESZERO.QTPRODDESTRE * (100 / (100 + C_PRODREGRAS_ACRESZERO.VRPEFATCOCPR));
                                 ELSE
                                    V_QTCALC := C_PRODREGRAS_ACRESZERO.QTPRODDESTRE / (100 / (100 + C_PRODREGRAS_ACRESZERO.VRPEFATCOCPR));
                                 END IF;
                              END IF;

                              IF V_IDQTLIQUBRUT = 'B' THEN
                                 V_QTLIQUIDA := C_PRODREGRAS_ACRESZERO.QTPRODDESTRE;
                                 IF (C_PRODREGRAS_ACRESZERO.VRPEFATCORPR IS NULL) OR (C_PRODREGRAS_ACRESZERO.VRPEFATCORPR = 0) THEN
                                    V_QTBRUTA := V_QTCALC;
                                 ELSE
                                    V_QTBRUTA := V_QTCALC * (100 / (100 - C_PRODREGRAS_ACRESZERO.VRPEFATCORPR));
                                 END IF;
                              ELSE
                                 V_QTBRUTA := C_PRODREGRAS_ACRESZERO.QTPRODDESTRE;
                                 IF (C_PRODREGRAS_ACRESZERO.VRPEFATCORPR IS NULL) OR (C_PRODREGRAS_ACRESZERO.VRPEFATCORPR = 0) THEN
                                    V_QTLIQUIDA := V_QTCALC;
                                 ELSE
                                    V_QTLIQUIDA := V_QTCALC / (100 / (100 - C_PRODREGRAS_ACRESZERO.VRPEFATCORPR));
                                 END IF;
                              END IF;

                              IF (C_PRODREGRAS_ACRESZERO.VRPEFATNUTPR IS NULL) OR (C_PRODREGRAS_ACRESZERO.VRPEFATNUTPR = 0) THEN
                                 V_QTCALCNUT := V_QTLIQUIDA;
                              ELSE
                                 V_QTCALCNUT := (V_QTLIQUIDA * C_PRODREGRAS_ACRESZERO.VRPEFATNUTPR) / 100;
                              END IF;
                           ELSE
                              V_QTLIQUIDA := C_PRODREGRAS_ACRESZERO.QTPRODPRAT;
                              V_QTBRUTA   := C_PRODREGRAS_ACRESZERO.QTBRUTAPR;
                              V_QTCALCNUT := C_PRODREGRAS_ACRESZERO.QTCALCNUTRPR;
                           END IF;

                           INSERT INTO RECEFILI
                              (CDFILAUXPRAT,
                               CDPRATO,
                               CDFILIAL,
                               CDPRODUTO,
                               QTPRODRECLOC, /*Quantidade lquida*/
                               VRPEFATCOCLO,
                               VRPEFATCORLO,
                               HRTEMPPREPLO,
                               SGUNIDADE,
                               QTUNIDESTLO,
                               QTBRUTALO,    /*Quantidade requerida*/
                               IDPRODBASELO,
                               DTULTATURECF,
                               VRPEFATNUTLO,
                               QTCALCNUTRLO,
                               QTBRUTALOUNR) /*Quantidade bruta*/
                           VALUES
                              (V_P,
                               C_PRODREGRAS_ACRESZERO.CDPRATO,
                               C_FILIAISAGREG.CDFILIAL,
                               C_PRODREGRAS_ACRESZERO.CDPRODDESTRE,
                               V_QTLIQUIDA,
                               C_PRODREGRAS_ACRESZERO.VRPEFATCOCPR,
                               C_PRODREGRAS_ACRESZERO.VRPEFATCORPR,
                               C_PRODREGRAS_ACRESZERO.HRTEMPPREPPR,
                               C_PRODREGRAS_ACRESZERO.SGUNIDADE,
                               V_QTBRUTA,/*Deve possuir o mesmo valor que a Quantidade requerida*/
                               V_QTBRUTA,
                               C_PRODREGRAS_ACRESZERO.IDPRODBASEPR,
                               SYSDATE,
                               C_PRODREGRAS_ACRESZERO.VRPEFATNUTPR,
                               V_QTCALCNUT,
                               V_QTBRUTA);
                     END;
                  END LOOP;

               END IF;

               V_CONTRECEF := 0;
               SELECT COUNT(*)
               INTO  V_CONTRECEF
               FROM  PRATFILI
               WHERE CDPRATO      = C_PRATOSORIG.CDPRATO AND
                     CDFILAUXPRAT = V_P AND
                     CDFILIAL     = C_FILIAISAGREG.CDFILIAL;

               IF (V_CONTRECEF > 0) THEN
                  DELETE FROM PRATFILI
                  WHERE
                     CDPRATO      = CDPRATO AND
                     CDFILAUXPRAT = V_P AND
                     CDFILIAL     = C_FILIAISAGREG.CDFILIAL;

                  INSERT INTO PRATFILI
                     (CDFILAUXPRAT,
                      CDPRATO,
                      CDFILIAL,
                      NRCORPRIFILI,
                      NRCORSEQFILI,
                      NRCORAUXFILI,
                      QTPADRAOFILI,
                      QTCONTAIFILI,
                      HRVALCONFILI,
                      NMFANTASIAPF,
                      QTCARBOPRFIL,
                      QTLIPIDPRFIL,
                      QTPROTEPRFIL,
                      QTCALORPRFIL,
                      QTALCOOPRFIL)
                  (SELECT
                      PT.CDFILAUXPRAT,
                      PT.CDPRATO,
                      RF.CDFILIAL,
                      PT.NRCORPRIPRAT,
                      PT.NRCORSEQPRAT,
                      PT.NRCORAUXPRAT,
                      SUM(QTPRODRECLOC) AS QTPRODRECLOC,
                      PT.QTCONTAIPRAT,
                      PT.HRVALCONPRAT,
                      PT.NMFANTASIAPR,
                      PT.QTCARBOPRPAD,
                      PT.QTLIPIDPRPAD,
                      PT.QTPROTEPRPAD,
                      PT.QTCALORPRPAD,
                      PT.QTALCOOPRPAD
                   FROM
                      PRATPADR PT,
                      RECEFILI RF
                   WHERE
                      PT.CDFILAUXPRAT = V_P                     AND
                      PT.CDPRATO      = C_PRATOSORIG.CDPRATO    AND
                      RF.CDFILIAL     = C_FILIAISAGREG.CDFILIAL AND
                      RF.CDFILAUXPRAT = PT.CDFILAUXPRAT         AND
                      RF.CDPRATO      = PT.CDPRATO
                   GROUP BY
                      PT.CDFILAUXPRAT,
                      PT.CDPRATO,
                      RF.CDFILIAL,
                      PT.NRCORPRIPRAT,
                      PT.NRCORSEQPRAT,
                      PT.NRCORAUXPRAT,
                      PT.QTCONTAIPRAT,
                      PT.HRVALCONPRAT,
                      PT.NMFANTASIAPR,
                      PT.QTCARBOPRPAD,
                      PT.QTLIPIDPRPAD,
                      PT.QTPROTEPRPAD,
                      PT.QTCALORPRPAD,
                      PT.QTALCOOPRPAD);
               END IF;
            END IF;

         END IF; /* REGRA APLICADA RECEITA PADRAO */

      END LOOP; /*FILIAIS AGREGADAS*/
   END LOOP; /*PRATOS*/
   COMMIT;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('INTEGRA_RECEITA_REGRAS','5.01.002'); END;
/

